"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importStar(require("react"));
var react_native_1 = require("react-native");
var useTimeout_1 = __importDefault(require("./useTimeout"));
var types_1 = require("./types");
var constants_1 = require("./constants");
exports.CollapsibleNavBarScrollView = react_1.memo(react_1.forwardRef(function (_a, ref) {
    var onChangeState = _a.onChangeState, animatedValue = _a.animatedValue, header = _a.header, headerMinHeight = _a.headerMinHeight, headerMaxHeight = _a.headerMaxHeight, _b = _a.initialState, initialState = _b === void 0 ? types_1.CollapsibleNavBarState.open : _b, _c = _a.useNativeDriver, useNativeDriver = _c === void 0 ? true : _c, style = _a.style, onLayout = _a.onLayout, props = __rest(_a, ["onChangeState", "animatedValue", "header", "headerMinHeight", "headerMaxHeight", "initialState", "useNativeDriver", "style", "onLayout"]);
    var _d = useTimeout_1.default(), setEndScrollTimeout = _d[0], clearEndScrollTimeout = _d[1];
    var _e = react_1.useState(constants_1.SCREEN_HEIGHT), containerHeight = _e[0], setContainerHeight = _e[1];
    var headerScrollDistance = react_1.useMemo(function () { return headerMaxHeight - headerMinHeight; }, []);
    var contentOffset = react_1.useMemo(function () { return ({
        x: 0,
        y: initialState === types_1.CollapsibleNavBarState.closed ? headerScrollDistance : 0,
    }); }, []);
    var contentHeight = react_1.useMemo(function () { return containerHeight - headerMinHeight; }, [containerHeight]);
    var localAnimatedValue = react_1.useMemo(function () { return new react_native_1.Animated.Value(initialState === types_1.CollapsibleNavBarState.closed ? headerScrollDistance : 0); }, []);
    var scrollY = react_1.useMemo(function () {
        if (animatedValue) {
            animatedValue.setValue(initialState === types_1.CollapsibleNavBarState.closed ? headerScrollDistance : 0);
            return animatedValue;
        }
        return localAnimatedValue;
    }, []);
    var scrollViewRef = react_1.useRef(null);
    var contentScrollViewRef = react_1.useRef(null);
    var scrollValue = react_1.useRef(0);
    var opened = react_1.useRef(true);
    var firstRender = react_1.useRef(true);
    var isDraging = react_1.useRef(false);
    var setOpened = react_1.useCallback(function (newOpened) {
        if (onChangeState && opened.current !== newOpened) {
            onChangeState(newOpened ? types_1.CollapsibleNavBarState.open : types_1.CollapsibleNavBarState.closed);
        }
        opened.current = newOpened;
    }, []);
    var open = react_1.useCallback(function (options) {
        setOpened(true);
        if (scrollViewRef.current) {
            var scrollParams = { y: 0, animated: options ? options === null || options === void 0 ? void 0 : options.animated : true };
            if (typeof scrollViewRef.current.scrollTo === 'function') {
                scrollViewRef.current.scrollTo(scrollParams);
            }
            else {
                // @ts-ignore
                scrollViewRef.current.getNode().scrollTo(scrollParams);
            }
        }
    }, [setOpened]);
    var close = react_1.useCallback(function (options) {
        setOpened(false);
        if (scrollViewRef.current) {
            var scrollParams = { animated: options ? options === null || options === void 0 ? void 0 : options.animated : true };
            if (typeof scrollViewRef.current.scrollToEnd === 'function') {
                scrollViewRef.current.scrollToEnd(scrollParams);
            }
            else {
                // @ts-ignore
                scrollViewRef.current.getNode().scrollToEnd(scrollParams);
            }
        }
    }, [setOpened]);
    var onScrollBegin = react_1.useCallback(function () {
        isDraging.current = true;
        clearEndScrollTimeout();
    }, []);
    var handleIntermediateState = react_1.useCallback(function () {
        clearEndScrollTimeout();
        if (scrollValue.current < headerScrollDistance) {
            if (scrollValue.current > headerScrollDistance / 2) {
                close();
            }
            else {
                open();
            }
        }
    }, [close, open]);
    var onMomentumScrollEnd = react_1.useCallback(function () {
        isDraging.current = false;
        if (scrollValue.current < headerScrollDistance) {
            setEndScrollTimeout(handleIntermediateState, 300);
        }
    }, [handleIntermediateState]);
    var onScrollEndDrag = react_1.useCallback(function (e) {
        var _a;
        isDraging.current = false;
        var velocity = ((_a = e.nativeEvent.velocity) === null || _a === void 0 ? void 0 : _a.y) || 0;
        if (velocity === 0 || (constants_1.isAndroid && Math.abs(Math.round(velocity)) <= 2)) {
            handleIntermediateState();
        }
    }, [handleIntermediateState]);
    var onScrollContentEndDrag = react_1.useCallback(function (event) {
        if (event.nativeEvent.contentOffset.y > 0 && opened.current) {
            close();
        }
    }, [close]);
    var handleLayout = react_1.useCallback(function (event) {
        if (firstRender.current && initialState === types_1.CollapsibleNavBarState.closed && constants_1.isAndroid) {
            close({ animated: false });
        }
        if (onLayout) {
            onLayout(event);
        }
        firstRender.current = false;
    }, [close]);
    var handleContainerLayout = react_1.useCallback(function (event) {
        var height = event.nativeEvent.layout.height;
        if (height !== containerHeight) {
            setContainerHeight(event.nativeEvent.layout.height);
        }
    }, [containerHeight]);
    var scrollToView = react_1.useCallback(function (viewRef, offset) {
        if (offset === void 0) { offset = 0; }
        if (viewRef && viewRef.measure) {
            viewRef.measure(function (ox, oy) {
                if (contentScrollViewRef.current) {
                    contentScrollViewRef.current.scrollTo({ y: oy + offset });
                }
            });
        }
    }, []);
    react_1.useImperativeHandle(ref, function () {
        if (contentScrollViewRef.current) {
            // @ts-ignore
            contentScrollViewRef.current.open = open;
            // @ts-ignore
            contentScrollViewRef.current.close = close;
            // @ts-ignore
            contentScrollViewRef.current.scrollToView = scrollToView;
            return contentScrollViewRef.current;
        }
        return null;
    }, [open, close, scrollToView]);
    react_1.useEffect(function () {
        scrollY.addListener(function (_a) {
            var value = _a.value;
            scrollValue.current = value;
            if (!isDraging.current) {
                clearEndScrollTimeout();
                if (value < headerScrollDistance) {
                    setEndScrollTimeout(handleIntermediateState, 300);
                }
            }
            setOpened(value < headerScrollDistance);
        });
        return function () {
            scrollY.removeAllListeners();
            scrollY.stopAnimation();
        };
    }, []);
    return (react_1.default.createElement(react_native_1.View, { style: styles.container, onLayout: handleContainerLayout },
        react_1.default.createElement(react_native_1.Animated.ScrollView, __assign({ bounces: false, showsVerticalScrollIndicator: false, onScroll: react_native_1.Animated.event([{ nativeEvent: { contentOffset: { y: scrollY } } }], {
                useNativeDriver: useNativeDriver,
            }), onScrollBeginDrag: onScrollBegin, onMomentumScrollBegin: onScrollBegin, onScrollEndDrag: onScrollEndDrag, onMomentumScrollEnd: onMomentumScrollEnd, scrollEventThrottle: 16, overScrollMode: 'never', contentOffset: contentOffset }, (typeof jest === 'undefined' ? { ref: scrollViewRef } : {})),
            react_1.default.createElement(react_native_1.View, { style: {
                    height: headerMaxHeight,
                } }, header),
            react_1.default.createElement(react_native_1.ScrollView, __assign({ ref: contentScrollViewRef, nestedScrollEnabled: true, bounces: false, onScrollEndDrag: onScrollContentEndDrag, style: [style, { height: contentHeight }], onLayout: handleLayout }, props)))));
}));
var styles = react_native_1.StyleSheet.create({
    container: {
        flex: 1,
    },
});
